﻿/*
 * These classes in namespace "Avi" have been written by
 * kbinani (Osaka, Japan)
 * kbinani@msn.com
 * 
 * Copyright (c) 2008, kbinani
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
namespace Boare.Lib.Avi {
    using System;
    using System.Drawing;
    using System.Drawing.Imaging;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;
    using System.IO;

    public class AviWriter {

        public static AVICOMPRESSOPTIONS DEFAULT_VIDEO_COMPRESS_OPTION {
            get {
                AVICOMPRESSOPTIONS result = new AVICOMPRESSOPTIONS();
                result.fccType = 0;
                result.fccHandler = 541215044;
                result.dwKeyFrameEvery = 0;
                result.dwQuality = 8500;
                result.dwBytesPerSecond = 0;
                result.dwFlags = 8;
                result.lpFormat = IntPtr.Zero;
                result.cbFormat = 0;
                result.lpParms = IntPtr.Zero;
                result.cbParms = 0;
                result.dwInterleaveEvery = 0;
                return result;
            }
        }

        private const int OF_READ = 0;
        private const int OF_READWRITE = 2;
        private const int OF_WRITE = 1;
        private const int OF_SHARE_COMPAT = 0;
        private const int OF_SHARE_DENY_NONE = 64;
        private const int OF_SHARE_DENY_READ = 48;
        private const int OF_SHARE_DENY_WRITE = 32;
        private const int OF_SHARE_EXCLUSIVE = 16;
        private const int OF_CREATE = 4096;

        [StructLayout( LayoutKind.Sequential, Pack = 1 )]
        private struct AVISTREAMINFOW {
            public UInt32 fccType, fccHandler, dwFlags, dwCaps;

            public UInt16 wPriority, wLanguage;

            public UInt32 dwScale, dwRate,
                             dwStart, dwLength, dwInitialFrames, dwSuggestedBufferSize,
                             dwQuality, dwSampleSize, rect_left, rect_top,
                             rect_right, rect_bottom, dwEditCount, dwFormatChangeCount;
            public UInt16 szName0, szName1, szName2, szName3, szName4, szName5,
                             szName6, szName7, szName8, szName9, szName10, szName11,
                             szName12, szName13, szName14, szName15, szName16, szName17,
                             szName18, szName19, szName20, szName21, szName22, szName23,
                             szName24, szName25, szName26, szName27, szName28, szName29,
                             szName30, szName31, szName32, szName33, szName34, szName35,
                             szName36, szName37, szName38, szName39, szName40, szName41,
                             szName42, szName43, szName44, szName45, szName46, szName47,
                             szName48, szName49, szName50, szName51, szName52, szName53,
                             szName54, szName55, szName56, szName57, szName58, szName59,
                             szName60, szName61, szName62, szName63;
        }


        // vfw.h
        [Serializable]
        [StructLayout( LayoutKind.Sequential, Pack = 1 )]
        public struct AVICOMPRESSOPTIONS {
            public UInt32 fccType;
            public UInt32 fccHandler;
            public UInt32 dwKeyFrameEvery;  // only used with AVICOMRPESSF_KEYFRAMES
            public UInt32 dwQuality;
            public UInt32 dwBytesPerSecond; // only used with AVICOMPRESSF_DATARATE
            public UInt32 dwFlags;
            public IntPtr lpFormat;
            public UInt32 cbFormat;
            public IntPtr lpParms;
            public UInt32 cbParms;
            public UInt32 dwInterleaveEvery;
            new public string ToString() {
                return "fccType=" + fccType + "\n" +
                    "fccHandler=" + fccHandler + "\n" +
                    "dwKeyFrameEvery=" + dwKeyFrameEvery + "\n" +
                    "dwQuality=" + dwQuality + "\n" +
                    "dwBytesPerSecond=" + dwBytesPerSecond + "\n" +
                    "dwFlags=" + dwFlags + "\n" +
                    "lpFormat=" + lpFormat + "\n" +
                    "cbFormat=" + cbFormat + "\n" +
                    "lpParms=" + lpParms + "\n" +
                    "cbParms=" + cbParms + "\n" +
                    "dwInterleaveEvery=" + dwInterleaveEvery;
            }
        }


        [StructLayout( LayoutKind.Sequential, Pack = 1 )]
        public struct BITMAPINFOHEADER {
            public UInt32 biSize;
            public Int32 biWidth;
            public Int32 biHeight;
            public Int16 biPlanes;
            public Int16 biBitCount;
            public UInt32 biCompression;
            public UInt32 biSizeImage;
            public Int32 biXPelsPerMeter;
            public Int32 biYPelsPerMeter;
            public UInt32 biClrUsed;
            public UInt32 biClrImportant;
        }


        public class AviException : ApplicationException {
            public AviException( string s )
                : base( s ) {
            }
            public AviException( string s, Int32 hr )
                : base( s ) {

                if ( hr == AVIERR_BADPARAM ) {
                    err_msg = "AVIERR_BADPARAM";
                } else {
                    err_msg = "unknown";
                }
            }

            public string ErrMsg() {
                return err_msg;
            }
            private const Int32 AVIERR_BADPARAM = -2147205018;
            private string err_msg;
        }


        public bool Open( string file_name, uint scale, uint rate, int width, int height, IntPtr hwnd ) {
            this._scale = scale;
            this._rate = rate;
            this.width_ = (UInt32)width;
            this.height_ = (UInt32)height;
            using ( Bitmap bmp = new Bitmap( width, height, PixelFormat.Format24bppRgb ) ) {
                BitmapData bmpDat = bmp.LockBits( new Rectangle( 0, 0, width, height ), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb );
                this.stride_ = (UInt32)bmpDat.Stride;
                bmp.UnlockBits( bmpDat );
            }
            AVIFileInit();
            int hr = AVIFileOpenW( ref pfile_, file_name, OF_WRITE | OF_CREATE, 0 );
            if ( hr != 0 ) {
                throw new AviException( "error for AVIFileOpenW" );
            }

            CreateStream();
            return SetOptions( hwnd );

        }


        [Obsolete]
        public bool Open( string fileName, float frameRate, int width, int height, IntPtr hwnd ) {
            this._scale = 1000;
            this._rate = (uint)(frameRate * this._scale);
            this.width_ = (UInt32)width;
            this.height_ = (UInt32)height;
            using ( Bitmap bmp = new Bitmap( width, height, PixelFormat.Format24bppRgb ) ) {
                BitmapData bmpDat = bmp.LockBits( new Rectangle( 0, 0, width, height ), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb );
                this.stride_ = (UInt32)bmpDat.Stride;
                bmp.UnlockBits( bmpDat );
            }
            AVIFileInit();
            int hr = AVIFileOpenW( ref pfile_, fileName, OF_WRITE | OF_CREATE, 0 );
            if ( hr != 0 ) {
                throw new AviException( "error for AVIFileOpenW" );
            }

            CreateStream();
            return SetOptions( hwnd );

        }


        /*public void DESTROYED_Open( string fileName, float frameRate, int width, int height, AVICOMPRESSOPTIONS compress ) {
            _scale = 1000;
            _rate = (uint)(frameRate * _scale);
            width_ = (UInt32)width;
            height_ = (UInt32)height;
            using ( Bitmap bmp = new Bitmap( width, height, PixelFormat.Format24bppRgb ) ) {
                BitmapData bmpDat = bmp.LockBits( new Rectangle( 0, 0, width, height ), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb );
                stride_ = (UInt32)bmpDat.Stride;
                bmp.UnlockBits( bmpDat );
            }
            AVIFileInit();
            int hr = AVIFileOpenW( ref pfile_, fileName, OF_WRITE | OF_CREATE, 0 );
            if ( hr != 0 ) {
                throw new AviException( "error for AVIFileOpenW" );
            }

            CreateStream();
            SetOptions( compress );
        }*/


        public void AddFrame( Bitmap bmp ) {
            BitmapData bmpDat = bmp.LockBits(
                new Rectangle( 0, 0, (int)width_, (int)height_ ), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb );

            int hr = AVIStreamWrite( m_video_compressed, count_, 1,
               bmpDat.Scan0, // pointer to data
               (Int32)(stride_ * height_),
               0, // 16 = AVIIF_KEYFRAMe
               0,
               0 );

            if ( hr != 0 ) {
                throw new AviException( "AVIStreamWrite" );
            }

            bmp.UnlockBits( bmpDat );

            count_++;
        }


        unsafe public static AVICOMPRESSOPTIONS RequireVideoCompressOption( AVICOMPRESSOPTIONS current_option ) {
            AviWriter temp = new AviWriter();
            temp._scale = 1000;
            temp._rate = 30 * temp._scale;
            int width = 10, height = 10;
            temp.width_ = (UInt32)width;
            temp.height_ = (UInt32)height;
            using ( Bitmap bmp = new Bitmap( width, height, PixelFormat.Format24bppRgb ) ) {
                BitmapData bmpDat = bmp.LockBits( new Rectangle( 0, 0, width, height ), ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb );
                temp.stride_ = (UInt32)bmpDat.Stride;
                bmp.UnlockBits( bmpDat );
            }
            AVIFileInit();
            string temp_file = Path.GetTempFileName() + ".avi";// .aviを付けないと、AVIFileOpenWが失敗する
            int hr = AVIFileOpenW( ref temp.pfile_, temp_file, OF_WRITE | OF_CREATE, 0 );
            if ( hr != 0 ) {
                throw new AviException( "error for AVIFileOpenW" );
            }

            temp.CreateStream();

            AVICOMPRESSOPTIONS opts = new AVICOMPRESSOPTIONS();
            opts.fccType = 0; //fccType_;
            opts.fccHandler = 0;//fccHandler_;
            opts.dwKeyFrameEvery = 0;
            opts.dwQuality = 0;  // 0 .. 10000
            opts.dwFlags = 0;  // AVICOMRPESSF_KEYFRAMES = 4
            opts.dwBytesPerSecond = 0;
            opts.lpFormat = new IntPtr( 0 );
            opts.cbFormat = 0;
            opts.lpParms = new IntPtr( 0 );
            opts.cbParms = 0;
            opts.dwInterleaveEvery = 0;

            opts = current_option;
            AVICOMPRESSOPTIONS* p = &opts;
            AVICOMPRESSOPTIONS** pp = &p;
            IntPtr x = temp.m_video;
            IntPtr* ptr_ps = &x;
            AVISaveOptions( IntPtr.Zero, 0, 1, ptr_ps, pp );
            //MessageBox.Show( "AVISaveOptions ok" );
            AVICOMPRESSOPTIONS copied = new AVICOMPRESSOPTIONS();
            copied = opts;
            AVIStreamRelease( temp.m_video );
            //MessageBox.Show( "AVIStreamRelease(temp.m_video) ok" );

            AVIFileRelease( temp.pfile_ );
            //MessageBox.Show( "AVIFileRelease ok" );
            AVIFileExit();
            //MessageBox.Show( "AVIFileExit ok" );
            File.Delete( temp_file );
            //MessageBox.Show( "File.Delete(fileName) ok" );
            return copied;
        }


        /// <summary>
        /// オーディオ圧縮の設定ダイアログを表示し、オーディオ圧縮の設定を取得します
        /// </summary>
        /// <returns></returns>
        unsafe public static AVICOMPRESSOPTIONS RequireAudioCompressOption() {
            string temp_file = Path.GetTempFileName();
            byte[] buf = new byte[] {
                82, 73, 70, 70, 94, 0, 0, 0, 87, 65, 86, 69, 102, 109, 116, 32, 16, 0, 0, 0, 1,
                0, 1, 0, 64, 31, 0, 0, 64, 31, 0, 0, 1, 0, 8, 0, 100, 97, 116, 97, 58, 0, 0, 0,
                128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
                128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
                128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
                128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
            };
            using ( FileStream fs = new FileStream( temp_file, FileMode.Create ) ) {
                fs.Write( buf, 0, buf.Length );
            }

            AVIFileInit();
            //MessageBox.Show( "AVIFileInit ok" );
            IntPtr audio;
            int hr = AVIStreamOpenFromFileW( out audio, temp_file, streamtypeAUDIO, 0, OF_READ, 0 );
            //MessageBox.Show( "AVIStreamOpenFromFileW ok" );

            AVICOMPRESSOPTIONS opts = new AVICOMPRESSOPTIONS();
            opts.fccType = streamtypeAUDIO; //fccType_;
            opts.fccHandler = 0;//fccHandler_;
            opts.dwKeyFrameEvery = 0;
            opts.dwQuality = 0;  // 0 .. 10000
            opts.dwFlags = 0;  // AVICOMRPESSF_KEYFRAMES = 4
            opts.dwBytesPerSecond = 0;
            opts.lpFormat = new IntPtr( 0 );
            opts.cbFormat = 0;
            opts.lpParms = new IntPtr( 0 );
            opts.cbParms = 0;
            opts.dwInterleaveEvery = 0;
            AVICOMPRESSOPTIONS* p = &opts;
            AVICOMPRESSOPTIONS** pp = &p;
            IntPtr x = audio;
            IntPtr* ptr_ps = &x;
            AVISaveOptions( IntPtr.Zero, 0, 1, ptr_ps, pp );
            //MessageBox.Show( "AVISaveOptions ok" );
            AVICOMPRESSOPTIONS copied = new AVICOMPRESSOPTIONS();
            copied = opts;
            AVIStreamRelease( audio );
            //MessageBox.Show( "AVIStreamRelease(audio) ok" );

            AVIFileExit();
            //MessageBox.Show( "AVIFileExit ok" );
            File.Delete( temp_file );
            return copied;
        }


        public void Close() {
            AVIStreamRelease( m_video );
            AVIStreamRelease( m_video_compressed );
#if DEBUG
            //AVIStreamRelease( m_audio );
            //AVIStreamRelease( m_audio_compressed );
#endif
            AVIFileRelease( pfile_ );
            AVIFileExit();
        }


        private void CreateStream() {
            // video stream
            AVISTREAMINFOW strhdr = new AVISTREAMINFOW();
            strhdr.fccType = streamtypeVIDEO;
            strhdr.fccHandler = fccHandler_;
            strhdr.dwFlags = 0;
            strhdr.dwCaps = 0;
            strhdr.wPriority = 0;
            strhdr.wLanguage = 0;
            strhdr.dwScale = _scale;
            strhdr.dwRate = _rate; // Frames per Second
            strhdr.dwStart = 0;
            strhdr.dwLength = 0;
            strhdr.dwInitialFrames = 0;
            strhdr.dwSuggestedBufferSize = height_ * stride_;
            strhdr.dwQuality = 0xffffffff; //-1;         // Use default
            strhdr.dwSampleSize = 0;
            strhdr.rect_top = 0;
            strhdr.rect_left = 0;
            strhdr.rect_bottom = height_;
            strhdr.rect_right = width_;
            strhdr.dwEditCount = 0;
            strhdr.dwFormatChangeCount = 0;
            strhdr.szName0 = 0;
            strhdr.szName1 = 0;
            int hr = AVIFileCreateStream( pfile_, out m_video, ref strhdr );
            if ( hr != 0 ) {
                throw new AviException( "AVIFileCreateStream; Video" );
            }
        }

        internal static void CalcScaleAndRate( decimal fps, out uint scale, out uint rate ) {
            scale = 100;
            rate = (uint)(fps * 1000m);
            int max = (int)(Math.Log10( uint.MaxValue ));
            for ( int i = 0; i <= max; i++ ) {
                scale = (uint)pow10( i );
                rate = (uint)(fps * scale);
                decimal t_fps = (decimal)rate / (decimal)scale;
                if ( t_fps == fps ) {
                    return;
                }
            }
        }

        private static int pow10( int x ) {
            int result = 1;
            for ( int i = 1; i <= x; i++ ) {
                result = result * 10;
            }
            return result;
        }

        unsafe private bool SetOptions( IntPtr hwnd ) {
            // VIDEO
            AVICOMPRESSOPTIONS opts = new AVICOMPRESSOPTIONS();
            opts.fccType = 0; //fccType_;
            opts.fccHandler = 0;//fccHandler_;
            opts.dwKeyFrameEvery = 0;
            opts.dwQuality = 0;  // 0 .. 10000
            opts.dwFlags = 0;  // AVICOMRPESSF_KEYFRAMES = 4
            opts.dwBytesPerSecond = 0;
            opts.lpFormat = new IntPtr( 0 );
            opts.cbFormat = 0;
            opts.lpParms = new IntPtr( 0 );
            opts.cbParms = 0;
            opts.dwInterleaveEvery = 0;

            AVICOMPRESSOPTIONS* p = &opts;
            AVICOMPRESSOPTIONS** pp;
            pp = &p;
            IntPtr x = m_video;
            IntPtr* ptr_ps;
            ptr_ps = &x;
            if ( AVISaveOptions( hwnd, 0, 1, ptr_ps, pp ) == 0 ) {
                return false;
            }
            int hr = AVIMakeCompressedStream( out m_video_compressed, m_video, ref opts, 0 );
            if ( hr != 0 ) {
                throw new AviException( "AVIMakeCompressedStream; Video" );
            }

            // TODO: AVISaveOptionsFree(...)

            BITMAPINFOHEADER bi = new BITMAPINFOHEADER();
            bi.biSize = 40;
            bi.biWidth = (Int32)width_;
            bi.biHeight = (Int32)height_;
            bi.biPlanes = 1;
            bi.biBitCount = 24;
            bi.biCompression = 0;  // 0 = BI_RGB
            bi.biSizeImage = stride_ * height_;
            bi.biXPelsPerMeter = 0;
            bi.biYPelsPerMeter = 0;
            bi.biClrUsed = 0;
            bi.biClrImportant = 0;

            hr = AVIStreamSetFormat( m_video_compressed, 0, ref bi, 40 );
            if ( hr != 0 ) {
                throw new AviException( "AVIStreamSetFormat", hr );
            }
            return true;
        }

        /*unsafe private void SetOptions( AVICOMPRESSOPTIONS compress ) {
            AVICOMPRESSOPTIONS opts = new AVICOMPRESSOPTIONS();
            opts = compress;
            int hr = AVIMakeCompressedStream( out m_video_compressed, m_video, ref opts, 0 );
            if ( hr != 0 ) {
                throw new AviException( "AVIMakeCompressedStream; Video" );
            }

            // TODO: AVISaveOptionsFree(...)

            BITMAPINFOHEADER bi = new BITMAPINFOHEADER();
            bi.biSize = 40;
            bi.biWidth = (Int32)width_;
            bi.biHeight = (Int32)height_;
            bi.biPlanes = 1;
            bi.biBitCount = 24;
            bi.biCompression = 0;  // 0 = BI_RGB
            bi.biSizeImage = stride_ * height_;
            bi.biXPelsPerMeter = 0;
            bi.biYPelsPerMeter = 0;
            bi.biClrUsed = 0;
            bi.biClrImportant = 0;

            hr = AVIStreamSetFormat( m_video_compressed, 0, ref bi, 40 );
            if ( hr != 0 ) {
                throw new AviException( "AVIStreamSetFormat", hr );
            }
        }*/


        [DllImport( "avifil32.dll" )]
        private static extern int AVIStreamOpenFromFileW(
            out IntPtr ppavi,
            [MarshalAs( UnmanagedType.LPWStr )]string szfile,
            uint fccType,
            int lParam,
            int mode,
            int dumy
        );

        [DllImport( "avifil32.dll" )]
        private static extern void AVIFileInit();

        [DllImport( "avifil32.dll" )]
        private static extern int AVIFileOpenW( ref int ptr_pfile, [MarshalAs( UnmanagedType.LPWStr )]string fileName, int flags, int dummy );

        [DllImport( "avifil32.dll" )]
        private static extern int AVIFileCreateStream(
          int ptr_pfile, out IntPtr ptr_ptr_avi, ref AVISTREAMINFOW ptr_streaminfo );

        [DllImport( "avifil32.dll" )]
        private static extern int AVIMakeCompressedStream(
          out IntPtr ppsCompressed, IntPtr aviStream, ref AVICOMPRESSOPTIONS ao, int dummy );

        [DllImport( "avifil32.dll" )]
        private static extern int AVIStreamSetFormat(
          IntPtr aviStream, Int32 lPos, ref BITMAPINFOHEADER lpFormat, Int32 cbFormat );

        /*[DllImport( "avifil32.dll" )]
        unsafe private static extern int AVISaveOptions(
          int hwnd, UInt32 flags, int nStreams, IntPtr* ptr_ptr_avi, AVICOMPRESSOPTIONS** ao );
        */
        [DllImport( "avifil32.dll" )]
        unsafe private static extern int AVISaveOptions(
          IntPtr hwnd, UInt32 flags, int nStreams, IntPtr* ptr_ptr_avi, AVICOMPRESSOPTIONS** ao );

        [DllImport( "avifil32.dll" )]
        private static extern int AVIStreamWrite(
          IntPtr aviStream, Int32 lStart, Int32 lSamples, IntPtr lpBuffer,
          Int32 cbBuffer, Int32 dwFlags, Int32 dummy1, Int32 dummy2 );

        [DllImport( "avifil32.dll" )]
        private static extern int AVIStreamRelease( IntPtr aviStream );

        [DllImport( "avifil32.dll" )]
        private static extern int AVIFileRelease( int pfile );

        [DllImport( "avifil32.dll" )]
        private static extern void AVIFileExit();

        private int pfile_ = 0;
        private IntPtr m_video = new IntPtr( 0 );
        private IntPtr m_video_compressed = new IntPtr( 0 );
        private IntPtr m_audio = new IntPtr( 0 );
        private IntPtr m_audio_compressed = new IntPtr( 0 );
        //private float frameRate_ = 0;
        private uint _scale;
        private uint _rate;
        private int count_ = 0;
        private UInt32 width_ = 0;
        private UInt32 stride_ = 0;
        private UInt32 height_ = 0 ;
        private static UInt32 streamtypeVIDEO = (UInt32)mmioFOURCC( 'v', 'i' , 'd', 's' ) ;
        private static UInt32 streamtypeAUDIO = (UInt32)mmioFOURCC( 'a', 'u', 'd', 's' );
        //private UInt32 fccType_ = 1935960438;  // vids
        private UInt32 fccHandler_ = 808810089;// IV50
        //1145656899;  // CVID
        //private Bitmap bmp_;
        private AVICOMPRESSOPTIONS m_aviCompressOptions;
        private bool m_option_setted = false;
        public AVICOMPRESSOPTIONS AviCompressOptions {
            get {
                return m_aviCompressOptions;
            }
            set {
                m_aviCompressOptions = value;
                m_option_setted = true;
            }
        }
        public static Int32 mmioFOURCC( char ch0, char ch1, char ch2, char ch3 ) {
            return ((Int32)(byte)(ch0) | ((byte)(ch1) << 8) | ((byte)(ch2) << 16) | ((byte)(ch3) << 24));
        }

    }
}

//public class AviA {

/*public const int StreamtypeVIDEO = 1935960438; //mmioStringToFOURCC("vids", 0)
public const int OF_SHARE_DENY_WRITE = 32;
public const int BMP_MAGIC_COOKIE = 19778; //ascii string "BM"

#region structure declarations

[StructLayout( LayoutKind.Sequential, Pack = 1 )]
public struct RECT {
    public UInt32 left;
    public UInt32 top;
    public UInt32 right;
    public UInt32 bottom;
}

[StructLayout( LayoutKind.Sequential, Pack = 1 )]
public struct BITMAPINFOHEADER {
    public UInt32 biSize;
    public Int32 biWidth;
    public Int32 biHeight;
    public Int16 biPlanes;
    public Int16 biBitCount;
    public UInt32 biCompression;
    public UInt32 biSizeImage;
    public Int32 biXPelsPerMeter;
    public Int32 biYPelsPerMeter;
    public UInt32 biClrUsed;
    public UInt32 biClrImportant;
}

[StructLayout( LayoutKind.Sequential, Pack = 1 )]
public struct AVISTREAMINFO {
    public UInt32 fccType;
    public UInt32 fccHandler;
    public UInt32 dwFlags;
    public UInt32 dwCaps;
    public UInt16 wPriority;
    public UInt16 wLanguage;
    public UInt32 dwScale;
    public UInt32 dwRate;
    public UInt32 dwStart;
    public UInt32 dwLength;
    public UInt32 dwInitialFrames;
    public UInt32 dwSuggestedBufferSize;
    public UInt32 dwQuality;
    public UInt32 dwSampleSize;
    public RECT rcFrame;
    public UInt32 dwEditCount;
    public UInt32 dwFormatChangeCount;
    [MarshalAs( UnmanagedType.ByValArray, SizeConst = 64 )]
    public UInt16[] szName;
}
[StructLayout( LayoutKind.Sequential, Pack = 1 )]
public struct BITMAPFILEHEADER {
    public Int16 bfType; //"magic cookie" - must be "BM"
    public Int32 bfSize;
    public Int16 bfReserved1;
    public Int16 bfReserved2;
    public Int32 bfOffBits;
}

#endregion structure declarations

#region method declarations

//Initialize the AVI library
[DllImport( "avifil32.dll" )]
public static extern void AVIFileInit();

//Open an AVI file
[DllImport( "avifil32.dll", PreserveSig = true )]
public static extern int AVIFileOpen(
    ref int ppfile,
    String szFile,
    int uMode,
    int pclsidHandler );

//Get a stream from an open AVI file
[DllImport( "avifil32.dll" )]
public static extern int AVIFileGetStream(
    int pfile,
    out IntPtr ppavi,
    int fccType,
    int lParam );

//Get the start position of a stream
[DllImport( "avifil32.dll", PreserveSig = true )]
public static extern int AVIStreamStart( int pavi );

//Get the length of a stream in frames
[DllImport( "avifil32.dll", PreserveSig = true )]
public static extern int AVIStreamLength( int pavi );

//Get information about an open stream
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamInfo(
    int pAVIStream,
    ref AVISTREAMINFO psi,
    int lSize );

//Get a pointer to a GETFRAME object (returns 0 on error)
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamGetFrameOpen(
    IntPtr pAVIStream,
    ref BITMAPINFOHEADER bih );

//[DllImport("avifil32.dll")]
//public static extern int AVIStreamGetFrameOpen(
//    IntPtr pAVIStream,
//    int dummy);

//Get a pointer to a packed DIB (returns 0 on error)
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamGetFrame(
    int pGetFrameObj,
    int lPos );

//Create a new stream in an open AVI file
[DllImport( "avifil32.dll" )]
public static extern int AVIFileCreateStream(
    int pfile,
    out IntPtr ppavi,
    ref AVISTREAMINFO ptr_streaminfo );

//Set the format for a new stream
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamSetFormat(
    IntPtr aviStream, Int32 lPos,
    ref BITMAPINFOHEADER lpFormat, Int32 cbFormat );

//Write a sample to a stream
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamWrite(
    IntPtr aviStream, Int32 lStart, Int32 lSamples,
    IntPtr lpBuffer, Int32 cbBuffer, Int32 dwFlags,
    Int32 dummy1, Int32 dummy2 );

//Release the GETFRAME object
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamGetFrameClose(
    int pGetFrameObj );

//Release an open AVI stream
[DllImport( "avifil32.dll" )]
public static extern int AVIStreamRelease( IntPtr aviStream );

//Release an open AVI file
[DllImport( "avifil32.dll" )]
public static extern int AVIFileRelease( int pfile );

//Close the AVI library
[DllImport( "avifil32.dll" )]
public static extern void AVIFileExit();

#endregion methos declarations

#region other useful avi functions

//public const int StreamtypeAUDIO = 1935963489; //mmioStringToFOURCC("auds", 0)
//public const int StreamtypeMIDI = 1935960429;  //mmioStringToFOURCC("mids", 0)
//public const int StreamtypeTEXT = 1937012852;  //mmioStringToFOURCC("txts", 0)
*/
/*[StructLayout(LayoutKind.Sequential, Pack=1)]
public struct AVIFILEINFO{
    public Int32 dwMaxBytesPerSecond;
    public Int32 dwFlags;
    public Int32 dwCaps;
    public Int32 dwStreams;
    public Int32 dwSuggestedBufferSize;
    public Int32 dwWidth;
    public Int32 dwHeight;
    public Int32 dwScale;
    public Int32 dwRate;
    public Int32 dwLength;
    public Int32 dwEditCount;
    [MarshalAs(UnmanagedType.ByValArray, SizeConst=64)]
    public char[] szFileType;
}*/

/*[StructLayout(LayoutKind.Sequential, Pack=1)]
public struct AVICOMPRESSOPTIONS {
    public UInt32   fccType;
    public UInt32   fccHandler;
    public UInt32   dwKeyFrameEvery;  // only used with AVICOMRPESSF_KEYFRAMES
    public UInt32   dwQuality;
    public UInt32   dwBytesPerSecond; // only used with AVICOMPRESSF_DATARATE
    public UInt32   dwFlags;
    public IntPtr   lpFormat;
    public UInt32   cbFormat;
    public IntPtr   lpParms;
    public UInt32   cbParms;
    public UInt32   dwInterleaveEvery;
}*/

/*[DllImport("avifil32.dll")]
public static extern int AVIMakeCompressedStream(
    out IntPtr ppsCompressed, IntPtr aviStream, 
    ref AVICOMPRESSOPTIONS ao, int dummy);*/

/*[DllImport("avifil32.dll")]
public static extern int AVISaveOptions(
    IntPtr hWnd,
    int uiFlags,
    int nStreams, 
    ref IntPtr ppavi,
    ref IntPtr ppOptions);*/

/*[DllImport("avifil32.dll")]
public static extern int AVIFileInfo(
    int pfile, 
    ref AVIFILEINFO pfi,
    int lSize);*/

/*[DllImport("winmm.dll", EntryPoint="mmioStringToFOURCCA")]
public static extern int mmioStringToFOURCC(String sz, int uFlags);*/

/*[DllImport("avifil32.dll")]
public static extern int AVIStreamRead(
    IntPtr pavi, 
    Int32 lStart,     
    Int32 lSamples,   
    IntPtr lpBuffer, 
    Int32 cbBuffer,   
    Int32  plBytes,  
    Int32  plSamples 
    );

#endregion other useful avi functions*/

//}
